#!/bin/ksh
###############################################################################
#
#       Capture Environment Information
#
#       Module  caputure_env.ksh
#
#       Based on capodbc supplied by DataDirect
#
#
#	This script captures various info about a users
#	Unix system and product installation and outputs
#	it to a text file that can be e-mailed to a rep.
#
#
###############################################################################
#
#       Maintenence log - insert most recent change descriptions at top
#
#	Date.... Request# WHO Description.........................................
#	07/04/09 00152835 QY  Check InstallDir and locale.
#
#	05/01/03 10084  PS  Added DSHOME check.
#	05/08/02 10080  PS  New.
###############################################################################


#	Get current working directory

	CE_PWD=`pwd`

#	output file that will be created

	CE_OUTFILE="$CE_PWD/capture_env.txt"

#	Print info about the file we are about to create

	echo "\nCreating file:-\n$CE_OUTFILE\c"
	echo "Original full path of file: $CE_OUTFILE\n" > $CE_OUTFILE

#	Record a time stamp

echo "Timestamp : `date`\n" >> $CE_OUTFILE

#	Get locale information
	echo "locale : `locale`\n" >> $CE_OUTFILE

#	Get information about the user's system and version
	echo "*** System information\n" >> $CE_OUTFILE
	echo "uname   : `uname -a`" >> $CE_OUTFILE
	echo "uptime  :`uptime`" >> $CE_OUTFILE

#	Get information about the current user

	myuid=`echo "test" > .testperm;ls -l .testperm|grep dsadm;rm -f .testperm`
	if [ -z "$myuid" ]
	then
		echo
		echo "ERROR: you need to be dsadm to run this script" | tee -a $logfile
		echo "\n*** Execution Terminated ***\n" | tee -a $logfile
	exit
	fi

	echo "\n*** User information\n" >> $CE_OUTFILE
	CE_USERID=`whoami`
	CE_GROUPS=`groups $CE_USERID`

	echo "User    : $CE_USERID" >> $CE_OUTFILE
	echo "Groups  : $CE_GROUPS" >> $CE_OUTFILE 
	echo "umask   : `umask`" >> $CE_OUTFILE

#	output the customers environment

	echo "\n*** Environment (using env command):\n" >> $CE_OUTFILE
	env >> $CE_OUTFILE

#	Get DataStage information

#	Get the DataStage Home Diectory & cd to the DstaStage directory

	DSHOME=`cat /.dshome`
	if [[ $DSHOME = "" ]]
	then
		echo "\n\nDSHOME not set, please enter the DSHOME path e.g"
		echo "/scratch/Ascential/DataStage/DSEngine\n"
		read DSHOME
	fi
	echo "\n\nDataStage home is  : $DSHOME" | tee -a $CE_OUTFILE


#	Get DataStage Release Information

	DSREL=`cat $DSHOME/.dsrel`
	echo "\nDataStage relese is  : $DSREL" | tee -a $CE_OUTFILE

#	Get dsenv information

	DSENV=`cat $DSHOME/dsenv`
	echo "\n\nDataStage dsenv file contents are  :-\n\n $DSENV" >> $CE_OUTFILE

#	Source dsenv
	. /$DSHOME/dsenv

# 	Now get ODBC info
#	Check to make sure ODBCINI is defined and then add that
#	to the output

	ODBCINFO=`cat $DSHOME/../branded_odbc/IBM_Tools/odbcinfo`

	echo "\n*** Current odbc driver set information\n" >> $CE_OUTFILE
	echo "$ODBCINFO\n" >> $CE_OUTFILE

	echo "\n*** Current ODBCINI\n" >> $CE_OUTFILE
	echo "ODBCINI : $ODBCINI\n" >> $CE_OUTFILE

	if [[ -n $ODBCINI && -f $ODBCINI ]] ; then

		cat $ODBCINI >> $CE_OUTFILE
	
#	 Also get the Install dir from ODBCINI

		CE_INSTALLDIR=`grep "InstallDir" $ODBCINI | awk -F= '{print $2}'`

	else

#	include a little error to let them know the ODBCINI wasn't set

		echo "\n\nERROR: The ODBCINI environment variable was not properly set."
		echo "       Information about the Connect ODBC installation will not be"
		echo "       included in the output file.\n"

		echo "No ODBCINI found (ODBCINI=$ODBCINI)" >> $CE_OUTFILE
		CE_INSTALLDIR=""
	fi

#	Capture the install dir if we got it from ODBCINI
	echo "\n*** Connect ODBC Directory structure and files\n" >> $CE_OUTFILE
	echo "InstallDir : $CE_INSTALLDIR\n" >> $CE_OUTFILE
	if [[ -n $CE_INSTALLDIR ]] ; then

		ls -la $CE_INSTALLDIR/.. >> $CE_OUTFILE
		ls -la $CE_INSTALLDIR/* >> $CE_OUTFILE
	else

		echo "Unable to determine InstallDir" >> $CE_OUTFILE
	fi

#	Print that we are done creating the file

	echo "[done...]\n"
	echo "\n*** End of $CE_OUTFILE" >> $CE_OUTFILE
